// Internal dependencies.
import * as sequences from './sequences';


export default {
  state: {
    item: null,
    snippet: '',
    saveState: null,
    progress: 0,
    content: {},
    context: '',
  },
  sequences,
};
